@@ -1,10 +1,11 @@
 __all__ = ["map_async", "starmap_async"]
-__version__ = "1.1"
+__version__ = "1.2.1"
 
 import importlib
 import itertools
 import multiprocessing
 import multiprocessing.pool as mp
+from typing import Sized, Iterable
 
 
 # gensim.utils.chunkize_serial
@@ -88,8 +89,8 @@ def chunkize(corpus, chunksize, maxsize=0):
             yield chunk
 
 
-def map_async(func, iterable, processes=None, scale=10, show_progress=False,
-              desc=None, starmap=False):
+def map_async(func, iterable: Iterable, processes=None, scale=10,
+              show_progress=False, desc=None, starmap=False) -> Iterable:
     """
     Apply function `func` on all items in an iterable with multiple processes.
 
@@ -117,7 +118,7 @@ def map_async(func, iterable, processes=None, scale=10, show_progress=False,
       starmap: (bool) whether to use `starmap` instead of `map`.
 
     Returns:
-      A list of transformed items (as long as the input iterable)
+      Iterable containing mapped items
 
     Example:
       >>> from map_async import map_async
@@ -130,19 +131,19 @@ def map_async(func, iterable, processes=None, scale=10, show_progress=False,
     """
     if processes is None:
         processes = multiprocessing.cpu_count()
-    chunks = list(chunkize(iterable, processes * scale))
+    size = len(iterable) if isinstance(iterable, Sized) else None
+    chunks = chunkize(iterable, processes * scale, maxsize=size)
     pool = mp.Pool(processes)
     map_func = pool.map if not starmap else pool.starmap
-    ret = []
     progress = None
     if show_progress:
         tqdm = importlib.import_module("tqdm")
-        progress = tqdm.tqdm(chunks, desc=desc)
+        progress = tqdm.tqdm(chunks, desc=desc, total=size)
     for chunk in chunks:
-        ret.extend(map_func(func, chunk))
+        items = map_func(func, chunk)
         if progress is not None:
-            progress.update(1)
-    return ret
+            progress.update(len(items))
+        yield from items
 
 
 def starmap_async(*args, **kwargs):
